import org.opencv.core.Core;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.imgproc.Imgproc;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;

import static org.opencv.imgproc.Imgproc.cvtColor;

public class MyOpenCvUtils {

    private static boolean isLibLoaded = false;

    public static synchronized void init() {
      if (!isLibLoaded) {
        System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
        isLibLoaded = true;
      }
    }

  static public BufferedImage fromGrayMat(Mat mat) {
    Mat mat2 = new Mat();
    cvtColor(mat, mat2, Imgproc.COLOR_GRAY2BGR);
    int pixels = mat.channels() * mat.cols() * mat.rows();
    byte[] pixelBytes = new byte[pixels * 3];
    mat2.get(0, 0, pixelBytes);
    BufferedImage bufferedImage = new BufferedImage(mat.cols(), mat.rows(), BufferedImage.TYPE_3BYTE_BGR);
    bufferedImage.getRaster().setDataElements(0, 0, mat.cols(), mat.rows(), pixelBytes);
    return convert(bufferedImage, BufferedImage.TYPE_INT_RGB);
  }


  static public BufferedImage convert(BufferedImage src, int newType) {
    BufferedImage newImage = new BufferedImage(src.getWidth(), src.getHeight(), newType);
    Graphics2D g = newImage.createGraphics();
    g.drawImage(src, 0, 0, null);
    return newImage;
  }

  static public Mat toGreyMat(BufferedImage image) {
    BufferedImage cvTypeBi = convert(image, BufferedImage.TYPE_3BYTE_BGR);
    Mat imageMat  = new Mat(image.getHeight(), image.getWidth(), CvType.CV_8UC3);
    imageMat.put(0, 0, ((DataBufferByte) cvTypeBi.getRaster().getDataBuffer()).getData());
    Mat greyMat   = new Mat(image.getHeight(), image.getWidth(), CvType.CV_8UC1);
    cvtColor(imageMat, greyMat, Imgproc.COLOR_BGR2GRAY);
    return greyMat;
  }

}
